"""Url router for the REST API
"""
from django.urls import re_path
from rest_framework.urlpatterns import format_suffix_patterns

import core_main_app.rest.web_page.views as web_page_views
import core_main_app.rest.xsl_transformation.views as xsl_transformation_views
from core_main_app.rest.blob import views as blob_views
from core_main_app.rest.data import views as data_views
from core_main_app.rest.template import views as template_views
from core_main_app.rest.template_version_manager import (
    views as template_version_manager_views,
)
from core_main_app.rest.template_xsl_rendering import (
    views as template_xsl_rendering_views,
)
from core_main_app.rest.user import views as user_views
from core_main_app.rest.workspace import views as workspace_views

urlpatterns = [
    re_path(
        r"^template-version-manager/global/$",
        template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
        name="core_main_app_rest_template_version_manager_global_list",
    ),
    re_path(
        r"^template-version-manager/user/$",
        template_version_manager_views.UserTemplateVersionManagerList.as_view(),
        name="core_main_app_rest_template_version_manager_user_list",
    ),
    re_path(
        r"^template-version-manager/(?P<pk>\w+)/$",
        template_version_manager_views.TemplateVersionManagerDetail.as_view(),
        name="core_main_app_rest_template_version_manager_detail",
    ),
    re_path(
        r"^template-version-manager/(?P<pk>\w+)/version/$",
        template_version_manager_views.TemplateVersion.as_view(),
        name="core_main_app_rest_template_version",
    ),
    re_path(
        r"^template-version-manager/(?P<pk>\w+)/disable/$",
        template_version_manager_views.DisableTemplateVersionManager.as_view(),
        name="core_main_app_rest_template_version_manager_disable",
    ),
    re_path(
        r"^template-version-manager/(?P<pk>\w+)/restore/$",
        template_version_manager_views.RestoreTemplateVersionManager.as_view(),
        name="core_main_app_rest_template_version_manager_restore",
    ),
    re_path(
        r"^template/version/(?P<pk>\w+)/current/$",
        template_version_manager_views.CurrentTemplateVersion.as_view(),
        name="core_main_app_rest_template_version_current",
    ),
    re_path(
        r"^template/version/(?P<pk>\w+)/disable/$",
        template_version_manager_views.DisableTemplateVersion.as_view(),
        name="core_main_app_rest_template_version_disable",
    ),
    re_path(
        r"^template/version/(?P<pk>\w+)/restore/$",
        template_version_manager_views.RestoreTemplateVersion.as_view(),
        name="core_main_app_rest_template_version_restore",
    ),
    re_path(
        r"^template/xsl_rendering/$",
        template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
        name="core_main_app_rest_template_detail",
    ),
    re_path(
        r"^template/xsl_rendering/(?P<pk>\w+)/$",
        template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
        name="core_main_app_rest_template_xsl_rendering",
    ),
    re_path(
        r"^template/global/$",
        template_version_manager_views.GlobalTemplateList.as_view(),
        name="core_main_app_rest_global_template_list",
    ),
    re_path(
        r"^template/user/$",
        template_version_manager_views.UserTemplateList.as_view(),
        name="core_main_app_rest_user_template_list",
    ),
    re_path(
        r"^template/(?P<pk>\w+)/download/$",
        template_views.TemplateDownload.as_view(),
        name="core_main_app_rest_template_download",
    ),
    re_path(
        r"^template/(?P<pk>\w+)/$",
        template_views.TemplateDetail.as_view(),
        name="core_main_app_rest_template_detail",
    ),
    re_path(
        r"^data/$", data_views.DataList.as_view(), name="core_main_app_rest_data_list"
    ),
    re_path(
        r"^data/permissions/$",
        data_views.DataPermissions.as_view(),
        name="core_main_app_rest_data_permissions",
    ),
    re_path(
        r"^data/download/(?P<pk>\w+)/$",
        data_views.DataDownload.as_view(),
        name="core_main_app_rest_data_download",
    ),
    re_path(
        r"^data/get-full$",
        data_views.get_by_id_with_template_info,
        name="core_main_app_rest_data_get_by_id_with_template_info",
    ),
    re_path(
        r"^data/query/keyword/$",
        data_views.ExecuteLocalKeywordQueryView.as_view(),
        name="core_main_app_rest_data_keyword_query",
    ),
    re_path(
        r"^data/query/$",
        data_views.ExecuteLocalQueryView.as_view(),
        name="core_main_app_rest_data_query",
    ),
    re_path(
        r"^data/(?P<pk>\w+)/assign/(?P<workspace_id>\w+)$",
        data_views.DataAssign.as_view(),
        name="core_main_app_rest_data_assign",
    ),
    re_path(
        r"^data/(?P<pk>\w+)/change-owner/(?P<user_id>\w+)$",
        data_views.DataChangeOwner.as_view(),
        name="core_main_app_rest_data_change_owner",
    ),
    re_path(
        r"^data/(?P<pk>\w+)/$",
        data_views.DataDetail.as_view(),
        name="core_main_app_rest_data_detail",
    ),
    re_path(
        r"^data/template/(?P<pk>\w+)/validate/$",
        data_views.Validation.as_view(),
        name="core_main_app_rest_data_validate",
    ),
    re_path(
        r"^data/template/(?P<pk>\w+)/migrate/$",
        data_views.Migration.as_view(),
        name="core_main_app_rest_data_migrate",
    ),
    re_path(
        r"^data/migration/task/(?P<task_id>[\w-]+)/progress/$",
        data_views.get_progress,
        name="core_main_app_rest_data_migration_task_progress",
    ),
    re_path(
        r"^data/migration/task/(?P<task_id>[\w-]+)/result/$",
        data_views.get_result,
        name="core_main_app_rest_data_migration_task_result",
    ),
    re_path(
        r"^admin/blob/$",
        blob_views.BlobListAdmin.as_view(),
        name="core_main_app_rest_blob_list_admin",
    ),
    re_path(
        r"^blob/$", blob_views.BlobList.as_view(), name="core_main_app_rest_blob_list"
    ),
    re_path(
        r"^blobs/delete/$",
        blob_views.BlobDeleteList.as_view(),
        name="core_main_app_rest_blob_delete_list",
    ),
    re_path(
        r"^blob/(?P<pk>\w+)/$",
        blob_views.BlobDetail.as_view(),
        name="core_main_app_rest_blob_detail",
    ),
    re_path(
        r"^blob/download/(?P<pk>\w+)/$",
        blob_views.BlobDownload.as_view(),
        name="core_main_app_rest_blob_download",
    ),
    re_path(
        r"^blob/(?P<pk>\w+)/assign/(?P<workspace_id>\w+)$",
        blob_views.BlobAssign.as_view(),
        name="core_main_app_rest_blob_assign",
    ),
    re_path(
        r"^xslt/$",
        xsl_transformation_views.XslTransformationList.as_view(),
        name="core_main_app_rest_xslt",
    ),
    re_path(
        r"^xslt/transform/$",
        xsl_transformation_views.XslTransformationTransform.as_view(),
        name="core_main_app_rest_xslt_transform",
    ),
    re_path(
        r"^xslt/(?P<pk>\w+)/$",
        xsl_transformation_views.XslTransformationDetail.as_view(),
        name="core_main_app_rest_xslt_detail",
    ),
    re_path(
        r"^workspace/$",
        workspace_views.WorkspaceList.as_view(),
        name="core_main_app_rest_workspace_list",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/detail/$",
        workspace_views.WorkspaceDetail.as_view(),
        name="core_main_app_rest_workspace_detail",
    ),
    re_path(
        r"^workspace/read_access",
        workspace_views.get_workspaces_with_read_access,
        name="core_main_app_rest_workspace_get_all_workspaces_with_read_access_by_user",
    ),
    re_path(
        r"^workspace/write_access",
        workspace_views.get_workspaces_with_write_access,
        name="core_main_app_rest_workspace_get_all_workspaces_with_write_access_by_user",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/is_public/$",
        workspace_views.is_workspace_public,
        name="core_main_app_rest_workspace_is_public",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/set_public/$",
        workspace_views.set_workspace_public,
        name="core_main_app_rest_workspace_set_public",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/set_private/$",
        workspace_views.set_workspace_private,
        name="core_main_app_rest_workspace_set_private",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/list_user_can_read/$",
        workspace_views.get_list_user_can_read_workspace,
        name="core_main_app_rest_workspace_list_user_can_read",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/list_user_can_write/$",
        workspace_views.get_list_user_can_write_workspace,
        name="core_main_app_rest_workspace_list_user_can_write",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/list_user_can_access/$",
        workspace_views.get_list_user_can_access_workspace,
        name="core_main_app_rest_workspace_list_user_can_access",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/list_group_can_read/$",
        workspace_views.get_list_group_can_read_workspace,
        name="core_main_app_rest_workspace_list_group_can_read",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/list_group_can_write/$",
        workspace_views.get_list_group_can_write_workspace,
        name="core_main_app_rest_workspace_list_group_can_write",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/list_group_can_access/$",
        workspace_views.get_list_group_can_access_workspace,
        name="core_main_app_rest_workspace_list_group_can_access",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/add_read_right_to_user/(?P<user_id>\w+)/$",
        workspace_views.add_user_read_right_to_workspace,
        name="core_main_app_rest_workspace_add_user_read",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/add_write_right_to_user/(?P<user_id>\w+)/$",
        workspace_views.add_user_write_right_to_workspace,
        name="core_main_app_rest_workspace_add_user_write",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/add_read_right_to_group/(?P<group_id>\w+)/$",
        workspace_views.add_group_read_right_to_workspace,
        name="core_main_app_rest_workspace_add_group_read",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/add_write_right_to_group/(?P<group_id>\w+)/$",
        workspace_views.add_group_write_right_to_workspace,
        name="core_main_app_rest_workspace_add_group_write",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/remove_read_right_to_user/(?P<user_id>\w+)/$",
        workspace_views.remove_user_read_right_to_workspace,
        name="core_main_app_rest_workspace_remove_user_read",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/remove_write_right_to_user/(?P<user_id>\w+)/$",
        workspace_views.remove_user_write_right_to_workspace,
        name="core_main_app_rest_workspace_remove_user_write",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/remove_read_right_to_group/(?P<group_id>\w+)/$",
        workspace_views.remove_group_read_right_to_workspace,
        name="core_main_app_rest_workspace_remove_group_read",
    ),
    re_path(
        r"^workspace/(?P<pk>\w+)/remove_write_right_to_group/(?P<group_id>\w+)/$",
        workspace_views.remove_group_write_right_to_workspace,
        name="core_main_app_rest_workspace_remove_group_write",
    ),
    re_path(
        r"^user/(?P<pk>[\w-]+)/$",
        user_views.UserDetail.as_view(),
        name="core_main_app_user_detail",
    ),
    re_path(
        r"^workspace/(?P<workspace_id>\w+)/data/$",
        data_views.DataListByWorkspace.as_view(),
        name="core_main_app_rest_data_workspace_list",
    ),
    re_path(
        r"^blob/(?P<pk>\w+)/change-owner/(?P<user_id>\w+)$",
        blob_views.BlobChangeOwner.as_view(),
        name="core_main_app_rest_blob_change_owner",
    ),
    re_path(
        r"^user/$", user_views.UserList.as_view(), name="core_main_app_user_detail"
    ),
    re_path(
        r"^template/xsl_rendering/(?P<pk>\w+)/set_default_xslt/(?P<xslt_id>\w+)/$",
        template_xsl_rendering_views.TemplateXslRenderingSetDefaultDetailXslt.as_view(),
        name="core_main_app_rest_template_xsl_rendering_set_default_xslt",
    ),
    re_path(
        r"^template/xsl_rendering/(?P<pk>\w+)/remove_detail_xslt/(?P<xslt_id>\w+)/$",
        template_xsl_rendering_views.TemplateXslRenderingRemoveDetailXslt.as_view(),
        name="core_main_app_rest_template_xsl_rendering_delete_detail_xslt",
    ),
    re_path(
        r"^template/xsl_rendering/(?P<pk>\w+)/add_detail_xslt/(?P<xslt_id>\w+)/$",
        template_xsl_rendering_views.TemplateXslRenderingAddDetailXslt.as_view(),
        name="core_main_app_rest_template_xsl_rendering_add_detail_xslt",
    ),
    re_path(
        r"^template/xsl_rendering/(?P<pk>\w+)/set_list_detail_xslt/$",
        template_xsl_rendering_views.TemplateXslRenderingSetListDetailXslt.as_view(),
        name="core_main_app_rest_template_xsl_rendering_set_list_detail_xslt",
    ),
    re_path(
        r"^login-page$",
        web_page_views.WebPageList.as_view(web_page_type="login"),
        name="login_page_rest_views",
    ),
    re_path(
        r"^admin/data/$",
        data_views.AdminDataList.as_view(),
        name="core_main_app_rest_admin_data_list",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



""" TemplateXslRendering API calls
"""
import logging

from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import ApiError
from core_main_app.components.template import api as template_api
from core_main_app.components.template_xsl_rendering.models import TemplateXslRendering

logger = logging.getLogger(__name__)


def add_or_delete(
    template_id,
    list_xslt,
    default_detail_xslt,
    list_detail_xslt,
    template_xsl_rendering_id=None,
):
    """Manage the saving of a TemplateXslRendering. If no XSLTs have been given,
    deletes the instance.

    Args:

        template_id: TemplateXslRendering.
        list_xslt: XSLT.
        default_detail_xslt: XSLT.
        list_detail_xslt:
        template_xsl_rendering_id: Instance id.

    Returns:

    """
    # Boolean to know if we need this instance in database, i.e there are XSLTs information.
    need_to_be_kept = (
        list_xslt is not None
        or default_detail_xslt is not None
        or list_detail_xslt is not None
    )

    if need_to_be_kept:
        return upsert(
            template_id,
            list_xslt,
            default_detail_xslt,
            list_detail_xslt,
            template_xsl_rendering_id,
        )
    else:
        try:
            if template_xsl_rendering_id:
                template_xsl_rendering = get_by_id(template_xsl_rendering_id)
                delete(template_xsl_rendering)

            return None
        except Exception:
            raise ApiError("An error occured while deleting the TemplateXSLRendering")


def set_list_detail_xslt(template_xsl_rendering, list_detail_xslt):
    """Set list of detail_xslt to a TemplateXslRendering

    Args:
        template_xsl_rendering
        list_detail_xslt

    Returns:

    """
    template_xsl_rendering.list_detail_xslt = list_detail_xslt
    _set_default_detail(template_xsl_rendering)
    template_xsl_rendering.save()


def add_detail_xslt(template_xsl_rendering, detail_xslt):
    """Add new detail xslt to a TemplateXslRendering

    Args:
        template_xsl_rendering
        detail_xslt

    Returns:

    """
    # Check if xslt exists in the list
    if detail_xslt not in template_xsl_rendering.list_detail_xslt:

        # Set the new xslt as default detail if the list is empty
        if not template_xsl_rendering.list_detail_xslt:
            template_xsl_rendering.default_detail_xslt = detail_xslt
        template_xsl_rendering.list_detail_xslt.append(detail_xslt)
        template_xsl_rendering.save()
    else:
        raise Exception("The xslt already exists in the list")


def delete_detail_xslt(template_xsl_rendering, detail_xslt):
    """Remove a detail_xslt from the details list. If this is a default detail xslt,
    set another default xslt.

    Args:
        template_xsl_rendering
        detail_xslt

    Returns:

    """

    if detail_xslt in template_xsl_rendering.list_detail_xslt:
        template_xsl_rendering.list_detail_xslt.remove(detail_xslt)

        # Check that removed detail xslt is not defined by default
        if detail_xslt == template_xsl_rendering.default_detail_xslt:
            _set_default_detail(template_xsl_rendering)

        template_xsl_rendering.save()
    else:
        raise Exception("The xslt does not exist in the list")


def set_default_detail_xslt(template_xsl_rendering, detail_xslt):
    """Set default detail_xslt

    Args:
        template_xsl_rendering
        detail_xslt

    Returns:

    """
    # Check if xslt exists in the list
    if detail_xslt in template_xsl_rendering.list_detail_xslt:
        template_xsl_rendering.default_detail_xslt = detail_xslt
        template_xsl_rendering.save()
    else:
        raise Exception("The xslt does not exist in the list")


def upsert(
    template_id,
    list_xslt,
    default_detail_xslt,
    list_detail_xslt,
    template_xsl_rendering_id=None,
):
    """Update or create a XSL Template rendering object

    Args:
        template_id:
        list_xslt:
        default_detail_xslt:
        list_detail_xslt:
        template_xsl_rendering_id:

    Returns:
        TemplateXSLRendering - The updated/created object.
    """
    try:
        template_xsl_rendering = get_by_id(template_xsl_rendering_id)
        template_xsl_rendering.list_xslt = list_xslt
        template_xsl_rendering.default_detail_xslt = default_detail_xslt
        template_xsl_rendering.list_detail_xslt = list_detail_xslt
    except Exception as exception:
        logger.warning(
            "Exception when saving TemplateXSLRendering object: %s" % str(exception)
        )
        template_xsl_rendering = TemplateXslRendering(
            template=template_id,
            list_xslt=list_xslt,
            default_detail_xslt=default_detail_xslt,
            list_detail_xslt=list_detail_xslt,
        )

    return _upsert(template_xsl_rendering)


def _upsert(template_xsl_rendering):
    """Upsert an TemplateXslRendering.

    Args:
        template_xsl_rendering: TemplateXslRendering instance.

    Returns:
        TemplateXslRendering instance

    """
    return template_xsl_rendering.save()


def delete(template_xsl_rendering):
    """Delete an TemplateXslRendering.

    Args:
        template_xsl_rendering: TemplateXslRendering to delete.

    """
    template_xsl_rendering.delete()


def get_by_id(template_xsl_rendering_id):
    """Get an TemplateXslRendering document by its id.

    Args:
        template_xsl_rendering_id: Id.

    Returns:
        TemplateXslRendering object.

    Raises:
        DoesNotExist: The TemplateXslRendering doesn't exist.
        ModelError: Internal error during the process.

    """
    return TemplateXslRendering.get_by_id(template_xsl_rendering_id)


def get_by_template_id(template_id):
    """Get TemplateXslRendering by its template id.

    Args:
        template_id: Template id.

    Returns:
        The TemplateXslRendering instance.

    """
    return TemplateXslRendering.get_by_template_id(template_id)


def get_by_template_hash(template_hash, request=None):
    """Get TemplateXslRendering by its template hash.

    Args:
        template_hash: Template hash.
        request:

    Returns:
        The TemplateXslRendering instance.

    """
    instance = None
    # Get list of templates with the given hash
    templates = template_api.get_all_accessible_by_hash(template_hash, request=request)
    # Check if one of the templates has a xslt. Stop when found.
    # FIXME: Check if this process is okay (no solutions to distinguish templates from same hash)
    for template in templates:
        try:
            instance = TemplateXslRendering.get_by_template_id(template.id)
            break
        except exceptions.DoesNotExist as e:
            logger.warning("get_by_template_hash threw an exception: ".format(str(e)))

    if instance is None:
        raise exceptions.DoesNotExist(
            "No TemplateXslRendering found with the given template hash"
        )

    return instance


def get_all():
    """Get all TemplateXslRendering.

    Returns:
        List of TemplateXslRendering.

    """
    return TemplateXslRendering.get_all()


def _set_default_detail(template_xsl_rendering):

    if template_xsl_rendering.list_detail_xslt:
        template_xsl_rendering.default_detail_xslt = (
            template_xsl_rendering.list_detail_xslt[0]
        )
    else:
        template_xsl_rendering.default_detail_xslt = None



""" Fixtures files for Data
"""
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.workspace.models import Workspace
from core_main_app.components.workspace import api as workspace_api
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class DataFixtures(FixtureInterface):
    """Data fixtures"""

    data_1 = None
    data_2 = None
    data_3 = None
    template = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.data_1 = Data(
            template=self.template, user_id="1", dict_content=None, title="title"
        ).save()
        self.data_2 = Data(
            template=self.template, user_id="2", dict_content=None, title="title2"
        ).save()
        self.data_3 = Data(
            template=self.template, user_id="1", dict_content=None, title="title3"
        ).save()
        self.data_collection = [self.data_1, self.data_2, self.data_3]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        template.content = xsd
        template.hash = ""
        template.filename = "filename"
        self.template = template.save()


class QueryDataFixtures(DataFixtures):
    """Data fixtures"""

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        content_1 = {
            "root": {
                "element": "value",
                "list": [{"element_list_1": 1}, {"element_list_2": 2}],
                "complex": {"child1": "test", "child2": 0},
            }
        }
        content_2 = {"root": {"element": "value2"}}
        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.data_1 = Data(
            template=self.template, user_id="1", dict_content=content_1, title="title"
        ).save()
        self.data_2 = Data(
            template=self.template, user_id="2", dict_content=content_2, title="title2"
        ).save()
        self.data_collection = [self.data_1, self.data_2]


class AccessControlDataFixture(FixtureInterface):
    """Access Control Data fixture"""

    USER_1_NO_WORKSPACE = 0
    USER_2_NO_WORKSPACE = 1
    USER_1_WORKSPACE_1 = 2
    USER_2_WORKSPACE_2 = 3

    template = None
    workspace_1 = None
    workspace_2 = None
    data_collection = None
    data_1 = None
    data_2 = None
    data_3 = None
    data_4 = None
    data_5 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        content = {"root": {"element": "value2"}}

        self.data_1 = Data(template=self.template, title="Data 1", user_id="1").save()
        self.data_2 = Data(template=self.template, title="Data 2", user_id="2").save()
        self.data_3 = Data(
            template=self.template,
            title="Data 3",
            user_id="1",
            workspace=self.workspace_1.id,
            dict_content=content,
        ).save()
        self.data_4 = Data(
            template=self.template,
            title="DataDoubleTitle",
            user_id="2",
            workspace=self.workspace_2.id,
        ).save()
        self.data_5 = Data(
            template=self.template,
            title="DataDoubleTitle",
            user_id="1",
            workspace=self.workspace_1.id,
        ).save()
        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_3,
            self.data_4,
            self.data_5,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        template.content = xsd
        template.hash = ""
        template.filename = "filename"
        self.template = template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1", owner="1", read_perm_id="1", write_perm_id="1"
        ).save()
        self.workspace_2 = Workspace(
            title="Workspace 2", owner="2", read_perm_id="2", write_perm_id="2"
        ).save()

    def generate_workspace_with_perm(self):
        """Generate the workspaces and the perm object.

        Returns:

        """
        try:
            self.workspace_1 = workspace_api.create_and_save("Workspace 1")
            self.workspace_2 = workspace_api.create_and_save("Workspace 2")
            self.data_3.workspace = self.workspace_1
            self.data_4.workspace = self.workspace_2
            self.data_5.workspace = self.workspace_1
        except Exception as e:
            print(str(e))


class DataMigrationFixture(FixtureInterface):
    """Data Template Fixture"""

    template_1 = None
    template_2 = None
    template_3 = None
    data_collection = None
    data_1 = None
    data_2 = None
    data_3 = None
    data_4 = None
    data_5 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        self.data_1 = Data(template=self.template_1, title="Data 1", user_id="1")
        self.data_1.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <test>test</test> \
                                  </root>'
        self.data_1.save()

        self.data_2 = Data(template=self.template_1, title="Data 2", user_id="1")
        self.data_2.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <test>test</test> \
                                  </root>'
        self.data_2.save()

        self.data_3 = Data(template=self.template_2, title="Data 3", user_id="1")
        self.data_3.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <test>test</test> \
                                  </root>'

        self.data_3.save()

        self.data_4 = Data(template=self.template_3, title="Data4", user_id="1")
        self.data_4.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <other>test</other> \
                                  </root>'
        self.data_4.save()

        self.data_5 = Data(template=self.template_3, title="Data5", user_id="1")
        self.data_5.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <other>test</other> \
                                  </root>'
        self.data_5.save()

        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_3,
            self.data_4,
            self.data_5,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        template1 = Template()
        template2 = Template()
        template3 = Template()
        xsd1 = '<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"> \
                <xsd:element name="root" type="simpleString"/> \
                <xsd:complexType name="simpleString"> \
                    <xsd:sequence> \
                    <xsd:element name="test" type="xsd:string"/></xsd:sequence> \
                </xsd:complexType> \
            </xsd:schema>'
        xsd2 = '<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"> \
                <xsd:element name="root" type="simpleString"/> \
                <xsd:complexType name="simpleString"> \
                    <xsd:sequence> \
                    <xsd:element name="test" type="xsd:string"/></xsd:sequence> \
                </xsd:complexType> \
            </xsd:schema>'
        xsd3 = '<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"> \
                <xsd:element name="root" type="simpleString"/> \
                <xsd:complexType name="simpleString"> \
                    <xsd:sequence> \
                    <xsd:element name="other" type="xsd:string"/></xsd:sequence> \
                </xsd:complexType> \
            </xsd:schema>'
        template1.content = xsd1
        template1.hash = ""
        template1.filename = "filename"
        template2.content = xsd2
        template2.hash = ""
        template2.filename = "filename"
        template3.content = xsd3
        template3.hash = ""
        template3.filename = "filename"
        self.template_1 = template1.save()
        self.template_2 = template2.save()
        self.template_3 = template2.save()



"""
Decode the date correctly for Django password
"""

import json
from datetime import datetime
from json import JSONDecoder

from django.utils import timezone

from core_main_app import settings


class DateTimeDecoder(json.JSONDecoder):
    def __init__(self, *args, **kargs):
        """

        Args:
            *args:
            **kargs:
        """
        JSONDecoder.__init__(self, object_hook=self.dict_to_object, *args, **kargs)

    def dict_to_object(self, d):
        """

        Args:
            d:

        Returns:

        """
        if "__type__" not in d:
            return d

        type = d.pop("__type__")
        try:
            if settings.USE_TZ:
                # timeit shows that datetime.now(tz=utc) is 24% slower
                date_obj = timezone.make_aware(datetime(**d), timezone.utc)
            else:
                return datetime(**d)

            return date_obj
        except:
            d["__type__"] = type
            return d



""" Authentication tests for Template Version Manager REST API
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.version_manager.models import VersionManager
from core_main_app.rest.template_version_manager import (
    views as template_version_manager_views,
)
from core_main_app.rest.template_version_manager.serializers import (
    TemplateVersionManagerSerializer,
    CreateTemplateSerializer,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGlobalTemplateVersionManagerListGetPermission(SimpleTestCase):
    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_anonymous_returns_http_403(
        self, template_version_manager_get_all_global_version_managers
    ):
        template_version_manager_get_all_global_version_managers.return_value = (
            TemplateVersionManager(user=None)
        )

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_authenticated_returns_http_200(
        self, template_version_manager_get_all_global_version_managers
    ):
        template_version_manager_get_all_global_version_managers.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_staff_returns_http_200(
        self, template_version_manager_get_all_global_version_managers
    ):
        template_version_manager_get_all_global_version_managers.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerListGetPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_all_version_manager_by_user_id")
    def test_authenticated_returns_http_200(
        self, template_version_manager_get_all_version_manager_by_user_id
    ):
        template_version_manager_get_all_version_manager_by_user_id.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_all_version_manager_by_user_id")
    def test_staff_returns_http_200(
        self, template_version_manager_get_all_version_manager_by_user_id
    ):
        template_version_manager_get_all_version_manager_by_user_id.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionManagerDetailGetPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(VersionManager, "get_by_id")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_authenticated_returns_http_200(
        self, template_version_manager_data, version_manager_get_by_id
    ):
        template_version_manager_data.return_value = True
        version_manager_get_by_id.return_value = VersionManager(user="1")

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(VersionManager, "get_by_id")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_200(
        self, template_version_manager_data, version_manager_get_by_id
    ):
        template_version_manager_data.return_value = True
        version_manager_get_by_id.return_value = VersionManager(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionPostPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(VersionManager, "get_by_id")
    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    def test_staff_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        version_manager_get_by_id,
    ):
        version_manager_get_by_id.return_value = VersionManager(user=None)
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(),
            mock_user,
            data={},
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestUserTemplateListPostPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_authenticated_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        create_template_version_manager_serializer_data,
        create_template_version_manager_serializer_save,
        create_template_version_manager_serializer_valid,
    ):
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        create_template_version_manager_serializer_data,
        create_template_version_manager_serializer_save,
        create_template_version_manager_serializer_valid,
    ):
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestGlobalTemplateListPostPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        create_template_version_manager_serializer_data,
        create_template_version_manager_serializer_save,
        create_template_version_manager_serializer_valid,
    ):
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestCurrentTemplateVersionPatchPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.set_current")
    @patch("core_main_app.components.version_manager.api.get_from_version")
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(
        self,
        template_get_by_id,
        version_manager_get_from_version,
        version_manager_set_current,
    ):
        version_manager_set_current.return_value = {}
        version_manager_get_from_version.return_value = VersionManager(user="1")
        template_get_by_id.return_value = Template(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDisableTemplateVersionPatchPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.disable_version")
    @patch("core_main_app.components.version_manager.api.get_from_version")
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(
        self,
        template_get_by_id,
        version_manager_get_from_version,
        version_manager_disable_version,
    ):
        version_manager_disable_version.return_value = VersionManager(user="1")
        version_manager_get_from_version.return_value = {}
        template_get_by_id.return_value = Template(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRestoreTemplateVersionPatchPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.restore_version")
    @patch("core_main_app.components.version_manager.api.get_from_version")
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(
        self,
        template_get_by_id,
        version_manager_get_from_version,
        version_manager_restore_version,
    ):
        version_manager_restore_version.return_value = {}
        version_manager_get_from_version.return_value = VersionManager(user="1")
        template_get_by_id.return_value = Template(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDisableTemplateVersionManagerPatchPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(VersionManager, "get_by_id")
    @patch("core_main_app.components.version_manager.api.disable")
    def test_authenticated_returns_http_200(
        self, version_manager_disable, version_manager_get_by_id
    ):
        version_manager_get_by_id.return_value = TemplateVersionManager(user="1")
        version_manager_disable.return_value = {}
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(VersionManager, "get_by_id")
    @patch("core_main_app.components.version_manager.api.disable")
    def test_staff_returns_http_200(
        self, version_manager_disable, version_manager_get_by_id
    ):
        version_manager_get_by_id.return_value = TemplateVersionManager(user="1")
        version_manager_disable.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRestoreTemplateVersionManagerPatchPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.restore")
    @patch.object(VersionManager, "get_by_id")
    def test_authenticated_returns_http_200(
        self, version_manager_get_by_id, version_manager_restore
    ):
        version_manager_restore.return_value = {}
        version_manager_get_by_id.return_value = TemplateVersionManager(user="1")
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.version_manager.api.restore")
    @patch.object(VersionManager, "get_by_id")
    def test_staff_returns_http_200(
        self, version_manager_get_by_id, version_manager_restore
    ):
        version_manager_restore.return_value = {}
        version_manager_get_by_id.return_value = TemplateVersionManager(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" XSD Flattener Requests URL Class
"""
from core_main_app.utils.requests_utils.requests_utils import send_get_request
from xml_utils.xsd_flattener.xsd_flattener_url import XSDFlattenerURL


class XSDFlattenerRequestsURL(XSDFlattenerURL):
    """Get the content of the dependency from the database or from the URL."""

    def __init__(self, xml_string, download_enabled=True):
        """Initializes the flattener

        Args:
            xml_string:
            download_enabled:
        """
        XSDFlattenerURL.__init__(
            self, xml_string=xml_string, download_enabled=download_enabled
        )

    def get_dependency_content(self, uri):
        """Get the content of the dependency from the URL using request util for HTTPS compliance

        Args:
            uri: Content URI

        Returns:
            Content

        """
        if self.download_enabled:
            dependency_file = send_get_request(uri)
            return dependency_file.content
        return ""



from unittest.case import TestCase

from bson.objectid import ObjectId
from django.core import exceptions as django_exceptions
from django.test import override_settings
from mock.mock import Mock, patch
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions
from core_main_app.components.template import api as template_api
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request


class TestTemplateGet(TestCase):
    @patch("core_main_app.components.template.models.Template.get_by_id")
    def test_template_get_returns_template(self, mock_get_by_id):
        # Arrange
        mock_template_filename = "Schema"
        mock_template_content = (
            "<schema xmlns='http://www.w3.org/2001/XMLSchema'></schema>"
        )

        mock_template = _create_mock_template(
            mock_template_filename, mock_template_content
        )

        mock_get_by_id.return_value = mock_template
        mock_user = create_mock_user("3", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        # Act
        result = template_api.get(mock_template.id, request=mock_request)

        # Assert
        self.assertIsInstance(result, Template)

    @patch("core_main_app.components.template.models.Template.get_by_id")
    def test_template_get_raises_exception_if_object_does_not_exist(
        self, mock_get_by_id
    ):
        # Arrange
        mock_absent_id = ObjectId()
        mock_get_by_id.side_effect = mongoengine_errors.DoesNotExist
        mock_user = create_mock_user("3")
        mock_request = create_mock_request(user=mock_user)

        # Act + Assert
        with self.assertRaises(mongoengine_errors.DoesNotExist):
            template_api.get(mock_absent_id, request=mock_request)


class TestTemplateGetAllByHash(TestCase):
    @patch("core_main_app.components.template.models.Template.get_all_by_hash")
    def test_template_get_all_by_hash_contains_only_template(
        self, mock_get_all_by_hash
    ):
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        _generic_get_all_test(
            self,
            mock_get_all_by_hash,
            template_api.get_all_accessible_by_hash("fake_hash", request=mock_request),
        )


class TestTemplateUpsert(TestCase):
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch("core_main_app.components.template.models.Template.save")
    def test_template_upsert_valid_returns_template(self, mock_save):
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        template = _create_template(
            filename="name.xsd",
            content="<schema xmlns='http://www.w3.org/2001/XMLSchema'></schema>",
        )

        mock_save.return_value = template
        result = template_api.upsert(template, request=mock_request)
        self.assertIsInstance(result, Template)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch("core_main_app.components.template.models.Template.save")
    def test_template_upsert_invalid_filename_raises_validation_error(self, mock_save):
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        template = _create_template(
            filename="1",
            content="<schema xmlns='http://www.w3.org/2001/XMLSchema'></schema>",
        )
        mock_save.side_effect = django_exceptions.ValidationError("")
        with self.assertRaises(django_exceptions.ValidationError):
            template_api.upsert(template, request=mock_request)

    @patch("core_main_app.components.template.models.Template.save")
    def test_template_upsert_invalid_content_raises_xsd_error(self, mock_save):
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        template = _create_template(filename="name.xsd", content="<schema></schema>")
        mock_save.return_value = None
        with self.assertRaises(exceptions.XSDError):
            template_api.upsert(template, request=mock_request)

    @patch("core_main_app.components.template.models.Template.save")
    def test_template_upsert_no_content_raises_error(self, mock_save):
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        template = _create_template(filename="name.xsd")
        mock_save.return_value = None
        with self.assertRaises(Exception):
            template_api.upsert(template, request=mock_request)


def _generic_get_all_test(self, mock_get_all, act_function):
    # Arrange
    mock_template_filename = "Schema"
    mock_template_content = "<schema xmlns='http://www.w3.org/2001/XMLSchema'></schema>"

    mock_template_1 = _create_mock_template(
        mock_template_filename, mock_template_content
    )
    mock_template_2 = _create_mock_template(
        mock_template_filename, mock_template_content
    )
    mock_get_all.return_value = [mock_template_1, mock_template_2]

    # Act
    result = act_function

    # Assert
    self.assertTrue(all(isinstance(item, Template) for item in result))


def _create_template(filename="", content=""):
    """
    Returns a template
    :param filename:
    :param content:
    :return:
    """
    return Template(id=ObjectId(), filename=filename, content=content)


def _create_mock_template(mock_template_filename="", mock_template_content=""):
    """
    Returns a mock template
    :param mock_template_filename:
    :param mock_template_content:
    :return:
    """
    mock_template = Mock(spec=Template)
    mock_template.filename = mock_template_filename
    mock_template.content = mock_template_content
    mock_template.id = ObjectId()
    return mock_template



""" Settings for core_main_app.

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""
import os
from os.path import join

from django.conf import settings

if not settings.configured:
    settings.configure()

CUSTOM_NAME = getattr(settings, "CUSTOM_NAME", "Local")
""" :py:class:`str`: Name of the local instance
"""

SERVER_URI = getattr(settings, "SERVER_URI", "http://127.0.0.1:8000")
""" :py:class:`str`: Server URI for import reference.
"""

INSTALLED_APPS = getattr(settings, "INSTALLED_APPS", [])
""" :py:class:`list`: List of apps installed.
"""

# Choose from:  black, black-light, blue, blue-light, green, green-light, purple, purple-light, red, red-light, yellow,
#               yellow-light
WEBSITE_ADMIN_COLOR = getattr(settings, "WEBSITE_ADMIN_COLOR", "yellow")
""" :py:class:`str`: color of the admin dashboard.
"""

# XML
XERCES_VALIDATION = getattr(settings, "XERCES_VALIDATION", False)
""" :py:class:`bool`: Enables Xerces validation (requires additional packages).
"""

# GridFS
GRIDFS_DATA_COLLECTION = getattr(settings, "GRIDFS_DATA_COLLECTION", "fs_data")
""" :py:class:`str`: Collection name for file storage in MongoDB.
"""

# Celery configuration
USE_BACKGROUND_TASK = getattr(settings, "USE_BACKGROUND_TASK", False)
""" :py:class:`bool`: Define use of celery for background tasks.
"""

# SMTP Configuration
USE_EMAIL = getattr(settings, "USE_EMAIL", False)
""" :py:class:`bool`: Activate email sending on the platform.
"""

SERVER_EMAIL = getattr(settings, "SERVER_EMAIL", "root@localhost")
""" :py:class:`str`: Email address sending the message.
"""

ADMINS = getattr(settings, "ADMINS", [])
""" :py:class:`list`: Email addresses of admins.
"""

MANAGERS = getattr(settings, "MANAGERS", [])
""" :py:class:`list`: Email addresses of moderators (managers). 
"""

EMAIL_SUBJECT_PREFIX = getattr(settings, "EMAIL_SUBJECT_PREFIX", "[CURATOR] ")
""" :py:class:`str`: Prefix for easy classification of emails.
"""

MONGO_HOST = getattr(settings, "MONGO_HOST", "localhost")
""" :py:class:`str`: MongoDB host.
"""

MONGO_PORT = getattr(settings, "MONGO_PORT", "27017")
""" :py:class:`str`: MongoDB port.
"""

MONGO_USER = getattr(settings, "MONGO_USER", "")
""" :py:class:`str`: MongoDB user.
"""

MONGO_PASSWORD = getattr(settings, "MONGO_PASSWORD", "")
""" :py:class:`str`: MongoDB password.
"""

MONGO_DB = getattr(settings, "MONGO_DB", "cdcs")
""" :py:class:`str`: MongoDB database.
"""

mongodb_uri_default = (
    f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_HOST}:{MONGO_PORT}/{MONGO_DB}"
)

MONGODB_URI = getattr(settings, "MONGODB_URI", mongodb_uri_default)
""" :py:class:`str`: MongoDB connection URI. Automatically generated from other parameters.
"""

USE_TZ = getattr(settings, "USE_TZ", True)
""" :py:class:`bool`: Activate use of timezones.
"""

SEARCHABLE_DATA_OCCURRENCES_LIMIT = getattr(
    settings, "SEARCHABLE_DATA_OCCURRENCES_LIMIT", None
)
""" :py:class:`int` | :py:attr:`None`: Limit for number of occurent to be returned by a search.
"""

BLOB_HOST = getattr(settings, "BLOB_HOST", "GRIDFS")
""" :py:class:`str`: Type of file storage.
"""

BLOB_HOST_URI = getattr(settings, "BLOB_HOST_URI", MONGODB_URI)
""" :py:class:`str`: File storage system URI.
"""

BLOB_HOST_USER = getattr(settings, "BLOB_HOST_USER", None)
""" :py:class:`str`: User for file storage.
"""

BLOB_HOST_PASSWORD = getattr(settings, "BLOB_HOST_PASSWORD", None)
""" :py:class:`str`: Password for file storage.
"""

# Lock
LOCK_OBJECT_TTL = getattr(settings, "LOCK_OBJECT_TTL", 600)  # 10 min
""" :py:class:`int`: Lock duration on files.
"""

# Results per page for paginator
RESULTS_PER_PAGE = getattr(settings, "RESULTS_PER_PAGE", 10)
""" :py:class:`int`: Results per page.
"""

CAN_SET_PUBLIC_DATA_TO_PRIVATE = getattr(
    settings, "CAN_SET_PUBLIC_DATA_TO_PRIVATE", True
)
""" :py:class:`bool`: Can set public data to private.
"""

DEFAULT_DATA_RENDERING_XSLT = getattr(
    settings,
    "DEFAULT_DATA_RENDERING_XSLT",
    join("core_main_app", "common", "xsl", "xml2html.xsl"),
)
""" :py:class:`str`: Path to default XSLT to render data.
"""

# Can anonymous access public document
CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = getattr(
    settings, "CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT", False
)
""" :py:class:`bool`: Can anonymous user access public document.
"""

DISPLAY_NIST_HEADERS = getattr(settings, "DISPLAY_NIST_HEADERS", False)
""" :py:class:`bool`: HTML pages show the NIST headers/footers.
"""

# Locale folder
BASE_DIR = getattr(
    settings, "BASE_DIR", os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
)
""" :py:class:`str`: Project installation directory.
"""

LOCALE_PATHS = getattr(
    settings, "LOCALE_PATHS", (os.path.join(BASE_DIR, "core_main_app/locale"),)
)
""" :py:class:`list`: Path for translation messages.
"""

CAN_SET_WORKSPACE_PUBLIC = getattr(settings, "CAN_SET_WORKSPACE_PUBLIC", True)
""" :py:class:`bool`: Can set workspace public.
"""

SSL_CERTIFICATES_DIR = getattr(settings, "SSL_CERTIFICATES_DIR", True)
""" :py:class:`str`: SSL certificates directory location.
"""

XSD_URI_RESOLVER = getattr(settings, "XSD_URI_RESOLVER", None)
""" :py:class:`str`: XSD URI Resolver for lxml validation. Choose from:  None, 'REQUESTS_RESOLVER'.
"""

XML_POST_PROCESSOR = getattr(settings, "XML_POST_PROCESSOR", "NUMERIC")
""" :py:class:`str`: post processor for xml to dict, choose between 'NUMERIC' and 'NUMERIC_AND_STRING' .
    'NUMERIC' convert numeric values from the xml document to integer or float.
    'NUMERIC_AND_STRING' convert numeric values and also store string representation .
"""

VERIFY_DATA_ACCESS = getattr(settings, "VERIFY_DATA_ACCESS", False)
""" :py:class:`bool`: Verify that data returned by a query can be accessed.
"""

DATA_SORTING_FIELDS = getattr(settings, "DATA_SORTING_FIELDS", [])
""" ::py:class:`str` Set the default sort fields for the data query. all the field must 
    be prefixed by "+" or "-" (asc or desc sort) the sort can be multi field and each 
    field must be delimited by "," (ex. ["-title","+name","+date"]) 
"""

AUTO_SET_PID = getattr(settings, "AUTO_SET_PID", False)
""" :py:class:`bool`: Enable PID auto-setting from core_linked_records_app.
"""

PASSWORD_RESET_DOMAIN_OVERRIDE = getattr(
    settings, "PASSWORD_RESET_DOMAIN_OVERRIDE", None
)
""" :py:class:`str`: Override domain of reset password email (e.g. localhost:8000)
"""



"""
Encode the date correctly for Django password
"""

from datetime import datetime
from json import JSONEncoder


class DateTimeEncoder(JSONEncoder):
    """Instead of letting the default encoder convert datetime to string,
    convert datetime objects into a dict, which can be decoded by the
    DateTimeDecoder
    """

    def default(self, obj):
        """

        Args:
            obj:

        Returns:

        """
        if isinstance(obj, datetime):
            return {
                "__type__": "datetime",
                "year": obj.year,
                "month": obj.month,
                "day": obj.day,
                "hour": obj.hour,
                "minute": obj.minute,
                "second": obj.second,
                "microsecond": obj.microsecond,
            }
        else:
            return JSONEncoder.default(self, obj)



""" Utils for template version manager rest Apis
"""
import json

from core_main_app.commons.exceptions import RestApiError


def load_dependencies(validated_data):
    """Return dependencies as a dict.

    Args:
        validated_data:

    Returns:

    """
    if "dependencies_dict" in validated_data:
        dependencies_dict = validated_data["dependencies_dict"]
        try:
            return json.loads(dependencies_dict)
        except:
            raise RestApiError("Incorrect format of the dependencies parameter.")
    return None



"""Pagination configuration for rest_framework
"""
from rest_framework.pagination import PageNumberPagination

from core_main_app.settings import RESULTS_PER_PAGE
from core_main_app.utils.pagination.mongoengine_paginator.paginator import (
    MongoenginePaginator,
)


class StandardResultsSetPagination(PageNumberPagination):
    page_size = RESULTS_PER_PAGE
    django_paginator_class = MongoenginePaginator



""" Abstract Data model
"""
from io import BytesIO

from django_mongoengine import fields, Document

from core_main_app.settings import (
    GRIDFS_DATA_COLLECTION,
    SEARCHABLE_DATA_OCCURRENCES_LIMIT,
)
from core_main_app.utils import xml as xml_utils
from core_main_app.utils.datetime_tools.utils import datetime_now
from core_main_app.utils.validation.regex_validation import not_empty_or_whitespaces
from mongoengine import errors as mongoengine_errors
from core_main_app.commons import exceptions


class AbstractData(Document):
    """AbstractData object"""

    dict_content = fields.DictField(blank=True)
    title = fields.StringField(blank=False, validation=not_empty_or_whitespaces)
    xml_file = fields.FileField(blank=False, collection_name=GRIDFS_DATA_COLLECTION)
    creation_date = fields.DateTimeField(blank=True, default=None)
    last_modification_date = fields.DateTimeField(blank=True, default=None)
    last_change_date = fields.DateTimeField(blank=True, default=None)

    _xml_content = None

    meta = {
        "abstract": True,
    }

    @property
    def xml_content(self):
        """Get xml content - read from a saved file.

        Returns:

        """
        # private field xml_content not set yet, and reference to xml_file to read is set
        if self._xml_content is None and self.xml_file is not None:
            # read xml file into xml_content field
            xml_content = self.xml_file.read()
            try:
                self._xml_content = (
                    xml_content.decode("utf-8") if xml_content else xml_content
                )
            except AttributeError:
                self._xml_content = xml_content
        # return xml content
        return self._xml_content

    @xml_content.setter
    def xml_content(self, value):
        """Set xml content - to be saved as a file.

        Args:
            value:

        Returns:

        """
        # update modification times
        self.last_modification_date = datetime_now()
        # update content
        self._xml_content = value

    def convert_and_save(self):
        """Save Data object and convert the xml to dict if needed.

        Returns:

        """
        self.convert_to_dict()
        self.convert_to_file()

        return self.save_object()

    def convert_to_dict(self):
        """Convert the xml contained in xml_content into a dictionary.

        Returns:

        """
        # transform xml content into a dictionary
        dict_content = xml_utils.raw_xml_to_dict(
            self.xml_content, xml_utils.post_processor
        )
        # if limit on element occurrences is set
        if SEARCHABLE_DATA_OCCURRENCES_LIMIT is not None:
            # Remove lists which size exceed the limit size
            xml_utils.remove_lists_from_xml_dict(
                dict_content, SEARCHABLE_DATA_OCCURRENCES_LIMIT
            )
        # store dictionary
        self.dict_content = dict_content

    def convert_to_file(self):
        """Convert the xml string into a file.

        Returns:

        """
        try:
            xml_file = BytesIO(self.xml_content.encode("utf-8"))
        except Exception:
            xml_file = BytesIO(self.xml_content)

        if self.xml_file.grid_id is None:
            # new file
            self.xml_file.put(xml_file, content_type="application/xml")
        else:
            # editing (self.xml_file gets a new id)
            self.xml_file.replace(xml_file, content_type="application/xml")

    def save_object(self):
        """Custom save. Set the datetime fields and save.

        Returns:

        """
        try:
            # initialize times
            now = datetime_now()
            # update change date every time the data is updated
            self.last_change_date = now
            if not self.id:
                # initialize when first created
                self.creation_date = now
                # initialize when first saved, then only updates when content is updated
                self.last_modification_date = now
            return self.save()
        except mongoengine_errors.NotUniqueError as e:
            raise exceptions.NotUniqueError(e)
        except Exception as ex:
            raise exceptions.ModelError(ex)



"""Admin AJAX views
"""
import html.parser
import json
from builtins import zip

from django.contrib.admin.views.decorators import staff_member_required
from django.http.response import HttpResponse, HttpResponseBadRequest
from django.urls import reverse_lazy
from django.utils.html import escape

from core_main_app.commons.exceptions import NotUniqueError
from core_main_app.components.template.api import init_template_with_dependencies
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.components.xsl_transformation import api as xsl_transformation_api
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.views.admin.forms import EditXSLTForm
from core_main_app.views.common.ajax import EditObjectModalView, DeleteObjectModalView


@staff_member_required
def resolve_dependencies(request):
    """Resolve import/includes to avoid local references.

    Args:
        request:

    Returns:

    """
    try:
        # Get the parameters
        name = request.POST.get("name", None)
        version_manager_id = request.POST.get("version_manager_id", "")
        filename = request.POST["filename"]
        xsd_content = request.POST["xsd_content"]
        schema_locations = request.POST.getlist("schemaLocations[]")
        dependencies = request.POST.getlist("dependencies[]")

        # create new object
        template = Template(
            filename=filename, content=_get_xsd_content_from_html(xsd_content)
        )
        init_template_with_dependencies(
            template,
            _get_dependencies_dict(schema_locations, dependencies),
            request=request,
        )

        # get the version manager or create a new one
        if version_manager_id != "":
            template_version_manager = version_manager_api.get(
                version_manager_id, request=request
            )
        else:
            template_version_manager = TemplateVersionManager(title=name)
        template_version_manager_api.insert(
            template_version_manager, template, request=request
        )
    except Exception as e:
        return HttpResponseBadRequest(escape(str(e)))

    return HttpResponse(json.dumps({}), content_type="application/javascript")


def _get_dependencies_dict(schema_locations, dependencies):
    """Build a dict from lists of schema locations and dependencies.

    Args:
        schema_locations:
        dependencies:

    Returns:

    """
    string_to_python_dependencies = []
    # transform 'None' into python None
    for dependency in dependencies:
        if dependency == "None":
            string_to_python_dependencies.append(None)
        else:
            string_to_python_dependencies.append(dependency)
    return dict(list(zip(schema_locations, string_to_python_dependencies)))


def _get_xsd_content_from_html(xsd_content):
    """Decode XSD content from HTML.

    Args:
        xsd_content:

    Returns:

    """
    html_parser = html.parser.HTMLParser()
    # FIXME: deprecated unescape
    xsd_content = html_parser.unescape(xsd_content)
    return xsd_content


class EditXSLTView(EditObjectModalView):
    form_class = EditXSLTForm
    document = XslTransformation
    success_url = reverse_lazy("admin:core_main_app_xslt")
    success_message = "XSLT edited with success."

    def _save(self, form):
        # Save treatment.
        try:
            xsl_transformation_api.upsert(self.object)
        except NotUniqueError:
            form.add_error(
                None,
                "An object with the same name already exists. Please choose "
                "another name.",
            )
        except Exception as e:
            form.add_error(None, str(e))


class DeleteXSLTView(DeleteObjectModalView):
    document = XslTransformation
    success_url = reverse_lazy("admin:core_main_app_xslt")
    success_message = "XSLT deleted with success."
    field_for_name = "name"

    def _delete(self, request, *args, **kwargs):
        # Delete treatment.
        xsl_transformation_api.delete(self.object)



""" Integration Test for Workspace Rest API
"""

from rest_framework import status

from core_main_app.components.workspace import api as workspace_api
from core_main_app.rest.workspace import views as workspace_rest_views
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    MongoIntegrationTransactionTestCase,
)
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.group.fixtures.fixtures import GroupFixtures
from tests.components.user.fixtures.fixtures import UserFixtures
from tests.components.workspace.fixtures.fixtures import WorkspaceFixtures

TITLE_1 = "title 1"
TITLE_2 = "title 2"
TITLE_3 = "title 3"

FAKE_WORKSPACE_ID = "507f1f77bcf86cd799439011"


class TestWorkspaceDetail(MongoIntegrationTransactionTestCase):
    """Test Workspace Detail"""

    def test_get_returns_http_200(self):
        # Context
        user = UserFixtures().create_user()

        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceDetail.as_view(),
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_workspace(self):
        # Context
        user = UserFixtures().create_user()

        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceDetail.as_view(),
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.data["title"], workspace.title)

    def test_get_wrong_id_returns_http_404(self):
        # Context
        user = UserFixtures().create_user()

        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceDetail.as_view(),
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_wrong_id_returns_http_404(self):
        # Context
        user = UserFixtures().create_user()

        # Act
        response = RequestMock.do_request_delete(
            workspace_rest_views.WorkspaceDetail.as_view(),
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_returns_http_204(self):
        # Context
        user = UserFixtures().create_user()

        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_delete(
            workspace_rest_views.WorkspaceDetail.as_view(),
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def test_delete_workspace(self):
        # Context
        user = UserFixtures().create_user()
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertEqual(len(workspace_api.get_all_by_owner(user)), 1)

        # Act
        response = RequestMock.do_request_delete(
            workspace_rest_views.WorkspaceDetail.as_view(),
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(len(workspace_api.get_all_by_owner(user)), 0)

    def test_delete_workspace_not_owner(self):
        # Context
        user = UserFixtures().create_user()
        user2 = UserFixtures().create_user(username="other")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertEqual(len(workspace_api.get_all_by_owner(user)), 1)

        # Act
        response = RequestMock.do_request_delete(
            workspace_rest_views.WorkspaceDetail.as_view(),
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestWorkspaceList(MongoIntegrationTransactionTestCase):
    """Test Workspace List"""

    def test_get_returns_http_200(self):
        # Context
        user = UserFixtures().create_user()

        WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        WorkspaceFixtures().create_workspace(user.id, TITLE_2)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceList.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_all_user_workspaces(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        other_user = UserFixtures().create_user(username="user2")

        WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        WorkspaceFixtures().create_workspace(user.id, TITLE_2)
        WorkspaceFixtures().create_workspace(other_user.id, TITLE_3)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceList.as_view(), user
        )

        # Assert
        self.assertEqual(len(response.data), 2)

    def test_get_all_workspaces_as_admin(self):
        # Context
        user = UserFixtures().create_super_user(username="user1")
        other_user = UserFixtures().create_user(username="user2")

        WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        WorkspaceFixtures().create_workspace(user.id, TITLE_2)
        WorkspaceFixtures().create_workspace(other_user.id, TITLE_3)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceList.as_view(), user
        )

        # Assert
        self.assertEqual(len(response.data), 3)

    def test_post_returns_http_201(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        mock_data = {"title": "title 1"}

        # Act
        response = RequestMock.do_request_post(
            workspace_rest_views.WorkspaceList.as_view(), user, data=mock_data
        )
        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_post_create_workspace(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        mock_data = {"title": TITLE_1}

        # Act
        RequestMock.do_request_post(
            workspace_rest_views.WorkspaceList.as_view(), user, data=mock_data
        )

        # Assert
        workspace = workspace_api.get_all_by_owner(user)
        self.assertEqual(len(workspace), 1)
        self.assertEqual(workspace[0].title, TITLE_1)
        self.assertEqual(workspace[0].owner, str(user.id))

    def test_post_create_workspace_with_owner(self):
        # Context
        FAKE_USER_ID = 123456
        user = UserFixtures().create_user(username="user1")
        mock_data = {"title": TITLE_1, "owner": FAKE_USER_ID}

        # Act
        response = RequestMock.do_request_post(
            workspace_rest_views.WorkspaceList.as_view(), user, data=mock_data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        workspace = workspace_api.get_all_by_owner(user)
        self.assertEqual(len(workspace), 1)
        self.assertEqual(workspace[0].title, TITLE_1)
        self.assertEqual(workspace[0].owner, str(user.id))
        self.assertNotEqual(workspace[0].owner, str(FAKE_USER_ID))

    def test_post_create_workspace_without_title(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        mock_data = {"owner": user.id}

        # Act
        response = RequestMock.do_request_post(
            workspace_rest_views.WorkspaceList.as_view(), user, data=mock_data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestWorkspaceReadAccess(MongoIntegrationTransactionTestCase):
    """Test Workspace Read Access"""

    def test_get_workspace_with_read_access_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_workspace_with_read_access_own_workspace(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, user
        )

        # Assert
        self.assertEqual(response.data[0]["title"], TITLE_1)

    def test_get_workspace_with_read_access_admin(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_super_user(username="user2")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, user2
        )

        # Assert
        self.assertEqual(response.data[0]["title"], TITLE_1)

    def test_get_workspace_with_read_access_other_workspace(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_read_access_to_workspace(workspace, user2, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, user2
        )

        # Assert
        self.assertEqual(response.data[0]["title"], TITLE_1)

    def test_get_workspace_with_read_access_public_workspace(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, user2
        )

        # Assert
        self.assertEqual(response.data[0]["title"], TITLE_1)


class TestWorkspaceWriteAccess(MongoIntegrationTransactionTestCase):
    """Test Workspace Write Access"""

    def test_get_workspace_with_write_access_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_workspace_with_write_access_own_workspace(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, user
        )

        # Assert
        self.assertEqual(response.data[0]["title"], TITLE_1)

    def test_get_workspace_with_write_access_admin(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_super_user(username="user2")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, user2
        )

        # Assert
        self.assertEqual(response.data[0]["title"], TITLE_1)

    def test_get_workspace_with_write_access_other_workspace(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_write_access_to_workspace(workspace, user2, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, user2
        )

        # Assert
        self.assertEqual(response.data[0]["title"], TITLE_1)

    def test_get_workspace_with_write_access_public_workspace(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, user2
        )

        # Assert
        self.assertEqual(len(response.data), 0)


class TestWorkspaceIsPublic(MongoIntegrationTransactionTestCase):
    """Test Workspace Is Public"""

    def test_is_workspace_public_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.is_workspace_public, user, param={"pk": workspace.id}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_is_workspace_public_return_false(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.is_workspace_public, user, param={"pk": workspace.id}
        )

        # Assert
        self.assertFalse(response.data)

    def test_is_workspace_public_return_true(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.is_workspace_public, user, param={"pk": workspace.id}
        )

        # Assert
        self.assertTrue(response.data)

    def test_is_workspace_public_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.is_workspace_public,
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestWorkspaceSetPublic(MongoIntegrationTransactionTestCase):
    """Test Workspace Set Public"""

    def test_set_workspace_public_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user, param={"pk": workspace.id}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_set_workspace_public_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user, param={"pk": workspace.id}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_set_workspace_public_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public,
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_set_workspace_public_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user, param={"pk": workspace.id}
        )

        # Assert
        workspace = workspace_api.get_by_id(workspace.id)
        self.assertTrue(workspace_api.is_workspace_public(workspace))

    def test_set_workspace_public_admin_not_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_super_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user2, param={"pk": workspace.id}
        )

        # Assert
        workspace = workspace_api.get_by_id(workspace.id)
        self.assertTrue(workspace_api.is_workspace_public(workspace))

    def test_set_workspace_public_user_not_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user2, param={"pk": workspace.id}
        )

        # Assert
        workspace = workspace_api.get_by_id(workspace.id)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

    def test_set_workspace_public_user_not_owner_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user2, param={"pk": workspace.id}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestWorkspaceSetPrivate(MongoIntegrationTransactionTestCase):
    """Test Workspace Set Private"""

    def test_set_workspace_private_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private, user, param={"pk": workspace.id}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_set_workspace_private_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private,
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_set_workspace_private_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)
        self.assertTrue(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private, user, param={"pk": workspace.id}
        )

        # Assert
        workspace = workspace_api.get_by_id(workspace.id)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

    def test_set_workspace_private_admin_not_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        user2 = UserFixtures().create_super_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)
        self.assertTrue(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private,
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        workspace = workspace_api.get_by_id(workspace.id)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

    def test_set_workspace_private_user_not_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)
        self.assertTrue(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private,
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        workspace = workspace_api.get_by_id(workspace.id)
        self.assertTrue(workspace_api.is_workspace_public(workspace))

    def test_set_workspace_private_user_not_owner_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertFalse(workspace_api.is_workspace_public(workspace))

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private,
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestWorkspaceListUserCanRead(MongoIntegrationTransactionTestCase):
    """Test Workspace List User Can read"""

    def test_get_list_user_can_read_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_list_user_can_read_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_list_user_can_read_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace,
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_get_list_user_can_read_workspace_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 0)

    def test_get_list_user_can_read_workspace_other_user(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_read_access_to_workspace(workspace, user2, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], user2.id)
        self.assertEquals(response.data[0]["username"], user2.username)

    def test_get_list_user_can_read_workspace_public(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 2)

    def test_get_list_user_can_read_workspace_other_user_admin(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        user3 = UserFixtures().create_super_user(username="user3")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_read_access_to_workspace(workspace, user2, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace,
            user3,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], user2.id)
        self.assertEquals(response.data[0]["username"], user2.username)


class TestWorkspaceListUserCanWrite(MongoIntegrationTransactionTestCase):
    """Test Workspace List User Can Write"""

    def test_get_list_user_can_write_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_list_user_can_write_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_list_user_can_write_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_get_list_user_can_write_workspace_owner(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 0)

    def test_get_list_user_can_write_workspace_other_user(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_write_access_to_workspace(workspace, user2, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], user2.id)
        self.assertEquals(response.data[0]["username"], user2.username)

    def test_get_list_user_can_write_workspace_public(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 0)

    def test_get_list_user_can_write_workspace_other_user_admin(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        user3 = UserFixtures().create_super_user(username="user3")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_write_access_to_workspace(workspace, user2, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user3,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], user2.id)
        self.assertEquals(response.data[0]["username"], user2.username)


class TestWorkspaceListGroupCanRead(MongoIntegrationTransactionTestCase):
    """Test Workspace List Group Can read"""

    def test_get_list_group_can_read_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_list_group_can_read_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_list_group_can_read_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_get_list_group_can_read_workspace_other_group(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_group_read_access_to_workspace(workspace, group, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], group.id)
        self.assertEquals(response.data[0]["name"], group.name)

    def test_get_list_group_can_read_workspace_public(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)
        GroupFixtures().create_group(name="group1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 3)

    def test_get_list_group_can_read_workspace_other_group_admin(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user3 = UserFixtures().create_super_user(username="user3")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        group = GroupFixtures().create_group(name="group1")
        workspace_api.add_group_read_access_to_workspace(workspace, group, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user3,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], group.id)
        self.assertEquals(response.data[0]["name"], group.name)


class TestWorkspaceListGroupCanWrite(MongoIntegrationTransactionTestCase):
    """Test Workspace List Group Can Write"""

    def test_get_list_group_can_write_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_list_group_can_write_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_list_group_can_write_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user2,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_get_list_group_can_write_workspace_other_group(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_group_write_access_to_workspace(workspace, group, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], group.id)
        self.assertEquals(response.data[0]["name"], group.name)

    def test_get_list_group_can_write_workspace_public(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        UserFixtures().add_publish_perm(user)
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.set_workspace_public(workspace, user)
        GroupFixtures().create_group(name="group1")

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            user,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 0)

    def test_get_list_group_can_write_workspace_other_group_admin(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        user3 = UserFixtures().create_super_user(username="user3")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_group_write_access_to_workspace(workspace, group, user)

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            user3,
            param={"pk": workspace.id},
        )

        # Assert
        self.assertEquals(len(response.data), 1)
        self.assertEquals(response.data[0]["id"], group.id)
        self.assertEquals(response.data[0]["name"], group.name)


class TestAddUserReadRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Add User Read Right To Workspace"""

    def test_add_user_read_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_add_user_read_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_read_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_add_user_read_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_read_right_to_workspace,
            user2,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_add_user_read_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertEqual(
            len(workspace_api.get_list_user_can_read_workspace(workspace, user)), 0
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_user_can_read_workspace(workspace, user)), 1
        )


class TestAddUserWriteRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Add User Write Right To Workspace"""

    def test_add_user_write_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_add_user_write_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_write_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_add_user_write_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_write_right_to_workspace,
            user2,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_add_user_write_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertEqual(
            len(workspace_api.get_list_user_can_write_workspace(workspace, user)), 0
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_user_can_write_workspace(workspace, user)), 1
        )


class TestAddGroupReadRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Add Group Read Right To Workspace"""

    def test_add_group_read_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_add_group_read_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_read_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_add_group_read_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_read_right_to_workspace,
            user2,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_add_group_read_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertEqual(
            len(workspace_api.get_list_group_can_read_workspace(workspace, user)), 0
        )

        # Act
        RequestMock.do_request_patch(
            workspace_rest_views.add_group_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_group_can_read_workspace(workspace, user)), 1
        )


class TestAddGroupWriteRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Add Group Write Right To Workspace"""

    def test_add_group_write_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_add_group_write_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_write_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_add_group_write_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_write_right_to_workspace,
            user2,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_add_group_write_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")

        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        self.assertEqual(
            len(workspace_api.get_list_group_can_write_workspace(workspace, user)), 0
        )

        # Act
        RequestMock.do_request_patch(
            workspace_rest_views.add_group_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_group_can_write_workspace(workspace, user)), 1
        )


class TestRemoveUserReadRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Remove User Read Right To Workspace"""

    def test_remove_user_read_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_remove_user_read_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_read_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_remove_user_read_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_read_right_to_workspace,
            user2,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_remove_user_read_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_read_access_to_workspace(workspace, user2, user)
        self.assertEqual(
            len(workspace_api.get_list_user_can_read_workspace(workspace, user)), 1
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_user_can_read_workspace(workspace, user)), 0
        )


class TestRemoveUserWriteRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Remove User Write Right To Workspace"""

    def test_remove_user_write_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_remove_user_write_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_write_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_remove_user_write_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_write_right_to_workspace,
            user2,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_remove_user_write_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_user_write_access_to_workspace(workspace, user2, user)
        self.assertEqual(
            len(workspace_api.get_list_user_can_write_workspace(workspace, user)), 1
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "user_id": user2.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_user_can_write_workspace(workspace, user)), 0
        )


class TestRemoveGroupReadRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Remove Group Read Right To Workspace"""

    def test_remove_group_read_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_remove_group_read_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_read_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_remove_group_read_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_read_right_to_workspace,
            user2,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_remove_group_read_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_group_read_access_to_workspace(workspace, group, user)
        self.assertEqual(
            len(workspace_api.get_list_group_can_read_workspace(workspace, user)), 1
        )

        # Act
        RequestMock.do_request_patch(
            workspace_rest_views.remove_group_read_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_group_can_read_workspace(workspace, user)), 0
        )


class TestRemoveGroupWriteRightToWorkspace(MongoIntegrationTransactionTestCase):
    """Test Remove Group Write Right To Workspace"""

    def test_remove_group_write_right_to_workspace_return_http_200(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_remove_group_write_right_to_workspace_return_http_404(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_write_right_to_workspace,
            user,
            param={"pk": FAKE_WORKSPACE_ID, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_remove_group_write_right_to_workspace_return_http_403(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_write_right_to_workspace,
            user2,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_remove_group_write_right_to_workspace_(self):
        # Context
        user = UserFixtures().create_user(username="user1")
        group = GroupFixtures().create_group(name="group1")
        workspace = WorkspaceFixtures().create_workspace(user.id, TITLE_1)
        workspace_api.add_group_write_access_to_workspace(workspace, group, user)
        self.assertEqual(
            len(workspace_api.get_list_group_can_write_workspace(workspace, user)), 1
        )

        # Act
        RequestMock.do_request_patch(
            workspace_rest_views.remove_group_write_right_to_workspace,
            user,
            param={"pk": workspace.id, "group_id": group.id},
        )

        # Assert
        self.assertEqual(
            len(workspace_api.get_list_group_can_write_workspace(workspace, user)), 0
        )



""" Template Version Manager Access Control
"""

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.utils.requests_utils.access_control import (
    get_request_from_args,
)


def can_write(func, *args, **kwargs):
    """Can write template.

    Args:
        func:
        args:
        kwargs:

    Returns:

    """
    request = get_request_from_args(*args, **kwargs)

    # super user
    if request.user.is_superuser:
        return func(*args, **kwargs)

    # anonymous cannot write
    if request.user.is_anonymous:
        raise AccessControlError("Template VM: The user doesn't have enough rights.")

    template_version_manager = next(
        (arg for arg in args if isinstance(arg, TemplateVersionManager)), None
    )

    # user is set
    if template_version_manager.user:
        if template_version_manager.user == str(request.user.id):
            return func(*args, **kwargs)
    # user is not set
    else:
        if request.user.is_staff:
            return func(*args, **kwargs)

    raise AccessControlError("Template VM: The user doesn't have enough rights.")
